package guru.kake.ronove

import javafx.collections.FXCollections
import javafx.scene.control.ComboBox
import javafx.scene.control.TextField
import tornadofx.*

/**
 * View for initializing new and editing existing projects' settings
 * @param project The project to edit
 */
class ProjectSettings(val project: Project) : Fragment("Ronove") {
    /** TornadoFX root */
    override val root = form {
        lateinit var nameField: TextField
        lateinit var fromField: ComboBox<String>
        lateinit var toField: ComboBox<String>

        val targetLanguages = FXCollections.observableArrayList(
                "BG - Bulgarian",
                "CS - Czech",
                "DA - Danish",
                "DE - German",
                "EL - Greek",
                "EN-GB - English (British)",
                "EN-US - English (American)",
                "ES - Spanish",
                "ET - Estonian",
                "FI - Finnish",
                "FR - French",
                "HU - Hungarian",
                "ID - Indonesian",
                "IT - Italian",
                "JA - Japanese",
                "KO - Korean",
                "LT - Lithuanian",
                "LV - Latvian",
                "NB - Norwegian (Bokmål)",
                "NL - Dutch",
                "PL - Polish",
                "PT-BR - Portuguese (Brazilian)",
                "PT-PT - Portuguese (non-Brazilian)",
                "RO - Romanian",
                "RU - Russian",
                "SK - Slovak",
                "SL - Slovenian",
                "SV - Swedish",
                "TR - Turkish",
                "UK - Ukrainian",
                "ZH - Chinese (simplified)"
        )
        
        val sourceLanguages = FXCollections.observableArrayList(
                "BG - Bulgarian",
                "CS - Czech",
                "DA - Danish",
                "DE - German",
                "EL - Greek",
                "EN - English",
                "ES - Spanish",
                "ET - Estonian",
                "FI - Finnish",
                "FR - French",
                "HU - Hungarian",
                "ID - Indonesian",
                "IT - Italian",
                "JA - Japanese",
                "KO - Korean",
                "LT - Lithuanian",
                "LV - Latvian",
                "NB - Norwegian (Bokmål)",
                "NL - Dutch",
                "PL - Polish",
                "PT - Portuguese",
                "RO - Romanian",
                "RU - Russian",
                "SK - Slovak",
                "SL - Slovenian",
                "SV - Swedish",
                "TR - Turkish",
                "UK - Ukrainian",
                "ZH - Chinese"
        )

        fieldset {
            field("Project name") {
                nameField = textfield(project.name)
            }
            /*field("Folder") {
                textfield()
                button("Open...")
            }*/
            hbox(20) {
                field("Language") {
                    fromField = combobox<String> {
                        items = sourceLanguages
                    }
                }
                field("to") {
                    toField = combobox<String> {
                        items = targetLanguages
                    }
                }
            }
        }
        fieldset {
            field {
                button("Cancel") { action { close() } }
                button("Confirm") {
                    action {
                        project.name = nameField.text
                        TranslationBackend.sourceLanguage = fromField.value.substringBefore(' ')
                        TranslationBackend.destinationLanguage = toField.value.substringBefore(' ')
                        close()
                    }
                }
            }
        }
    }


}