package guru.kake.ronove

import tornadofx.*
import kotlin.system.exitProcess

/**
 * The splash screen show on program startup, allowing the user to create a new project or load an existing
 * one
 * @param project the Project object the data of a new empty project or the chosen loaded project will be
 * stored into
 */
class SplashScreen(project: Project) : View("Ronove") {
    /** TornadoFX root */
    override val root = borderpane {
        center = form {
            fieldset {
                for (recent in Project.recent) {
                    field {
                        hyperlink(recent.first) {
                            action {
                                project.loadFile(recent.second)
                                (app as Ronove).table.refresh()
                                close()
                            }
                        }
                    }
                }
            field {
                button("New project") { action {
                    project.new()
                    (app as Ronove).table.refresh()
                    close()
                }}
                button("Open...") { action {
                    project.open()
                    (app as Ronove).table.refresh()
                    close()
                }}
                button("Exit") { action { exitProcess(0) }}
            }
            }
        }
        right = imageview("/ronove300.png") {
            fitHeight = 200.0
            isPreserveRatio = true
        }
        visibleProperty().onChange {
            currentWindow?.sizeToScene()
        }
    }

    override fun onDock() {
        modalStage?.width = 450.0
    }
}