package guru.kake.ronove

import javafx.scene.control.TableView
import tornadofx.*

/**
 * The central view of the main window used to shew the table of the translation data
 * @param ronove reference to the main app
 */
class TranslationPageView(val ronove: Ronove) : View() {
    init {
        PageJumpMenu.pageView = this
    }

    private lateinit var tableView: TableView<Line>
    /** TornadoFX fragment for page chaning buttons */
    lateinit var pageJumps: PageJumpMenu

    /** TornadoFX root */
    override val root = borderpane {
        tableView = tableview {
            isEditable = true

            column("Original", Line::original).makeEditable()
            column("Machine translation", Line::machineTranslation).makeEditable()
            column("Final", Line::finalTranslation).makeEditable()
            column("Notes", Line::notes).makeEditable()

            contextmenu {}
        }
        ronove.table = tableView
        tableView.selectionModel.isCellSelectionEnabled = true
        /*tableView.selectionModel.selectedItemProperty().addListener(fun(_, _, newSelection) {
            if (newSelection != null) {

                val row: Int = tableView.selectionModel.selectedCells[0].row
                val col = tableView.columns[tableView.selectionModel.selectedCells[0].column]
                Platform.runLater { tableView.edit(row, col) }
            }
        })
        tableView.onKeyPressed = EventHandler { event: KeyEvent ->
            val pos: TablePosition<*, *>? = tableView.focusModel.focusedCell
            if (pos != null && event.code.isLetterKey) {
                @Suppress("UNCHECKED_CAST")
                tableView.edit(pos.row, pos.tableColumn as TableColumn<Line, *>?)
            }
        }*/
        center = tableView

        pageJumps = PageJumpMenu()
        bottom = pageJumps.root
    }

    init {ronove.cellSelection = tableView.selectionModel.selectedItemProperty()}

    /**
     * Loads the currently selected page to the table view
     */
    private fun refresh() {
        tableView.items =
            ronove.project.pages[ronove.currentPage.value-1].lines.asObservable()
        tableView.refresh()
    }

    init {
        ronove.currentPage.addListener { _ -> refresh(); pageJumps.refresh()}
        ronove.pageCount.addListener { _ -> pageJumps.refresh()}
    }
}

/**
 * Fragment of the main window implementing the page switcher at the bottom
 */
class PageJumpMenu : Fragment() {
    companion object {
        internal lateinit var pageView: TranslationPageView
    }

    /**
     * Refresh the page switcher, updating the selected page and number of pages
     */
    fun refresh() {
        val menu = PageJumpMenu()
        replaceWith(menu)
        pageView.pageJumps = menu
    }

    /** TornadoFX root */
    override val root = hbox {
        addClass(RonoveStylesheet.vcenter)
        button("<<") {action {pageView.ronove.changePage(1)}}
        button("<") {action { pageView.ronove.changePage(pageView.ronove.currentPage.value-1)}}
        for (i in 1..pageView.ronove.pageCount.get()) {
            button("$i") {action {pageView.ronove.changePage(i)}}
        }
        menubutton("...") {
            for (i in 1..pageView.ronove.pageCount.get()) {
                item("$i") {action {pageView.ronove.changePage(i)}}
            }
        }
        button(">") {action { pageView.ronove.changePage(pageView.ronove.currentPage.value+1)}}
        button(">>") {action { pageView.ronove.changePage(pageView.ronove.pageCount.get())}}
        button("+") {
            action {
                pageView.ronove.addPage()
            }
        }
    }
}