import guru.kake.ronove.*
import javafx.stage.Modality
import javafx.stage.Stage
import javafx.stage.StageStyle
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith
import org.testfx.api.FxRobot
import org.testfx.framework.junit5.ApplicationExtension
import org.testfx.framework.junit5.Start
import org.testfx.framework.junit5.Stop
import tornadofx.*
import kotlin.test.assertEquals

class DummyMainWindow : View("Test") {
    override val root = pane {
        label("Dummy")
    }
}

@ExtendWith(ApplicationExtension::class)
class TranslationPageViewTest {

    private val ronove = Ronove()
    private lateinit var dummy: DummyMainWindow
    private lateinit var fragment: TranslationPageView

    @Suppress("Unused")
    @Start private fun start(stage: Stage) {
        dummy = DummyMainWindow()
        dummy.openModal(stageStyle = StageStyle.UTILITY, block = false,
            modality = Modality.APPLICATION_MODAL, owner = stage)
        fragment = TranslationPageView(ronove)
        fragment.openModal(stageStyle = StageStyle.UTILITY, block = false,
            modality = Modality.APPLICATION_MODAL, owner = stage)
        assertEquals(1,ronove.pageCount.get())
        for (i in 1..5) {
            ronove.addPage()
        }
        assertEquals(6,ronove.pageCount.get())

    }

    @Test fun pageJumpTest(robot: FxRobot) {
        robot.clickOn("2");  assertEquals(2,ronove.currentPage.value)
        robot.clickOn("1");  assertEquals(1,ronove.currentPage.value)
        robot.clickOn("4");  assertEquals(4,ronove.currentPage.value)
        robot.clickOn("3");  assertEquals(3,ronove.currentPage.value)
        robot.clickOn("5");  assertEquals(5,ronove.currentPage.value)
        robot.clickOn("<");  assertEquals(4,ronove.currentPage.value)
        robot.clickOn("<");  assertEquals(3,ronove.currentPage.value)
        robot.clickOn("<");  assertEquals(2,ronove.currentPage.value)
        robot.clickOn(">>"); assertEquals(6,ronove.currentPage.value)
        robot.clickOn("<<"); assertEquals(1,ronove.currentPage.value)
        robot.clickOn(">");  assertEquals(2,ronove.currentPage.value)
        robot.clickOn(">");  assertEquals(3,ronove.currentPage.value)
        robot.clickOn(">");  assertEquals(4,ronove.currentPage.value)
        robot.clickOn("+");  assertEquals(7,ronove.pageCount.get())
        robot.clickOn("+");  assertEquals(8,ronove.pageCount.get())
    }

    @Test fun edit(robot: FxRobot) {
        ronove.project.pages[2].lines.add(Line("a","b","c"))
        ronove.project.pages[2].lines.add(Line("d","e","f"))
        ronove.project.pages[2].lines.add(Line("g","h","i"))
        robot.clickOn("3")
        robot.doubleClickOn("f"); robot.write("foxtrot\n")
        robot.doubleClickOn("e"); robot.write("echo\n")
        robot.doubleClickOn("i"); robot.write("india\n")
        robot.doubleClickOn("h"); robot.write("hotel\n")
        robot.doubleClickOn("d"); robot.write("delta\n")
        robot.doubleClickOn("b"); robot.write("bravo\n")
        robot.doubleClickOn("a"); robot.write("alpha\n")
        robot.doubleClickOn("g"); robot.write("golf\n")
        robot.doubleClickOn("c"); robot.write("charlie\n")
        assertEquals("[alpha, delta, golf]", ronove.project.pages[2].getColumn(Line::original).toString())
        assertEquals("[bravo, echo, hotel]", ronove.project.pages[2].getColumn(Line::machineTranslation).toString())
        assertEquals("[charlie, foxtrot, india]", ronove.project.pages[2].getColumn(Line::finalTranslation).toString())
    }

    @Suppress("Unused")
    @Stop fun stop() {
        fragment.close()
        dummy.close()
    }
}