package guru.kake.xmlp

import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Test
import kotlin.test.assertFailsWith

class XMLParserTest {

    private val xml: String ="""
        <tag>
            <tag><cont foo="not" bar="not">one</cont><cont></cont></tag>
            <tag><attr foo="two" bar="not">not</attr></tag>
            <tag><tag><both foo="not" bar="three">four</both></tag></tag>
            <attr foo="five" />
        </tag>
        """.trimIndent()

    private val eof: String ="""
        <tag><tag><tag></tag></tag>
        """.trimIndent()

    private val close: String ="""
        <tag></tag></tag>
        """.trimIndent()


    @Test fun parseXML() {
        val sb = StringBuilder()
        XMLParser.parse(xml.byteInputStream()) {
            element("cont") {
                start { sb.append(content()?:"") }
            }
            element("attr") {
                start { sb.append(attribute("foo"))}
            }
            element("both") {
                start {
                    sb.append(content())
                    sb.append(attribute("bar"))
                }
            }
            element("tag") {
                start {
                    sb.append("{")
                }
                end{
                    sb.append("}")
                }
            }
        }
        assertEquals(sb.toString(),"{{one}{two}{{fourthree}}five}")
    }

    @Test fun malformed() {
        assertFailsWith<com.ctc.wstx.exc.WstxEOFException> { XMLParser.parse(eof.byteInputStream()) {}}
        assertFailsWith<com.ctc.wstx.exc.WstxParsingException> { XMLParser.parse(close.byteInputStream()) {}}
    }
}